

// ------------------------------------------------------------------------------
//
// LudumDare.com 48hr compo, July 2002
//
// Dave Reed (Bluescreen)'s entry
//
// 'OpenGL blitting' code
//
// ------------------------------------------------------------------------------

#include <windows.h>
#include <gl\gl.h>
#include <math.h>

#include "winmain.h"
#include "sprites.h"


float MATH_normalizeVector(VECTOR *vIn,VECTOR *vOut)
{
	float l;
	
	l=(float)sqrt((vIn->x*vIn->x)+(vIn->y*vIn->y)+(vIn->z*vIn->z));

	if (l!=0.0f)
	{
		vOut->x=vIn->x/l;
		vOut->y=vIn->y/l;
		vOut->z=vIn->z/l;
	}
	return l;
}


// ------------------------------------------------------------------------------
// Init a sprite struct - for simpler blitting later
// ------------------------------------------------------------------------------

void SPRITE_init(SPRITE *frame, float width3d,float height3d,float handleX,float handleY,
				 int tex,int texWidth,int texHeight,int texX,int texY,int pixWidth,int pixHeight)
{

	frame->width =width3d;
	frame->height=height3d;
	
	frame->ox=handleX;
	frame->oy=handleY;

	frame->tex=tex;
	

	frame->u[0]=((float)(texX))/((float)(texWidth));
	frame->v[2]=((float)(texY))/((float)(texHeight));
	frame->u[2]=(((float)(texX))+((float)(pixWidth)))/((float)(texWidth));
	frame->v[0]=(((float)(texY))+((float)(pixHeight)))/((float)(texHeight));

	frame->v[2]=1.0f-frame->v[2];
	frame->v[0]=1.0f-frame->v[0];

	frame->u[1]=frame->u[2];
	frame->v[1]=frame->v[0];
	frame->u[3]=frame->u[0];
	frame->v[3]=frame->v[2];

	
}


// ------------------------------------------------------------------------------
// Blit a sprite - with scaling and rotation, and a 3D position :)
// ------------------------------------------------------------------------------

void SPRITE_blit(SPRITE *frame, VECTOR *pos,float rot,float scale)
{


	glPushMatrix();

	GL_bindTexture(frame->tex);
	
	glTranslatef(pos->x,pos->y,pos->z);	
	glRotatef(rot,0,0,1.0f);
	glScalef(scale,scale,scale);

	glBegin(GL_TRIANGLE_FAN);
	
	glTexCoord2f(frame->u[0],frame->v[0]);
	glVertex3f(-frame->ox,-frame->oy,0.0f);

	glTexCoord2f(frame->u[1],frame->v[1]);
	glVertex3f(-frame->ox+frame->width,-frame->oy,0.0f);

	glTexCoord2f(frame->u[2],frame->v[2]);
	glVertex3f(-frame->ox+frame->width,-frame->oy+frame->height,0.0f);
	
	glTexCoord2f(frame->u[3],frame->v[3]);
	glVertex3f(-frame->ox,-frame->oy+frame->height,0.0f);

	glEnd();

	glPopMatrix();
}







// ------------------------------------------------------------------------------
// Blits a tile - with front/top/bot/left/right faces
//
// Copy+paste coding nastiness.....
//
// ------------------------------------------------------------------------------

void SPRITE_blitTile(SPRITE *frame, VECTOR *pos,float rot,float scale,float frontZ, float backZ,float r,float g,float b)
{

	float depth=-3.0f;

	glPushMatrix();

	GL_bindTexture(frame->tex);
	
	glTranslatef(pos->x,pos->y,pos->z);	
	glRotatef(rot,0,0,1.0f);
	glScalef(scale,scale,scale);

	
	



	glColor3f(r*0.6f,g*0.6f,b*0.6f);

	// Top
	glBegin(GL_TRIANGLE_FAN);

	glTexCoord2f(frame->u[1],frame->v[1]);
	glVertex3f(-frame->ox+frame->width,-frame->oy,frontZ);
	
	glTexCoord2f(frame->u[0],frame->v[0]);
	glVertex3f(-frame->ox,-frame->oy,frontZ);
	
	glTexCoord2f(frame->u[3],frame->v[3]);
	glVertex3f(-frame->ox,-frame->oy,backZ);

	glTexCoord2f(frame->u[2],frame->v[2]);	
	glVertex3f(-frame->ox+frame->width,-frame->oy,backZ);			
		
	glEnd();
	
	
	// Bot
	glBegin(GL_TRIANGLE_FAN);
	
	glTexCoord2f(frame->u[0],frame->v[0]);
	glVertex3f(-frame->ox,-frame->oy+frame->height,frontZ);

	glTexCoord2f(frame->u[1],frame->v[1]);
	glVertex3f(-frame->ox+frame->width,-frame->oy+frame->height,frontZ);
	
	glTexCoord2f(frame->u[2],frame->v[2]);	
	glVertex3f(-frame->ox+frame->width,-frame->oy+frame->height,backZ);			
	
	glTexCoord2f(frame->u[3],frame->v[3]);
	glVertex3f(-frame->ox,-frame->oy+frame->height,backZ);
	
	glEnd();

	
	glColor3f(r*0.4f,g*0.4f,b*0.4f);

	// Right
	glBegin(GL_TRIANGLE_FAN);
	
	glTexCoord2f(frame->u[0],frame->v[0]);
	glVertex3f(-frame->ox,-frame->oy,frontZ);

	glTexCoord2f(frame->u[1],frame->v[1]);
	glVertex3f(-frame->ox,-frame->oy+frame->height,frontZ);

	glTexCoord2f(frame->u[2],frame->v[2]);
	glVertex3f(-frame->ox,-frame->oy+frame->height,backZ);

	glTexCoord2f(frame->u[3],frame->v[3]);
	glVertex3f(-frame->ox,-frame->oy,backZ);
	
	glEnd();


	// Left
	glBegin(GL_TRIANGLE_FAN);
	
	glTexCoord2f(frame->u[0],frame->v[0]);
	glVertex3f(-frame->ox+frame->width,-frame->oy+frame->height,frontZ);

	glTexCoord2f(frame->u[1],frame->v[1]);
	glVertex3f(-frame->ox+frame->width,-frame->oy,frontZ);	

	glTexCoord2f(frame->u[2],frame->v[2]);
	glVertex3f(-frame->ox+frame->width,-frame->oy,backZ);

	glTexCoord2f(frame->u[3],frame->v[3]);
	glVertex3f(-frame->ox+frame->width,-frame->oy+frame->height,backZ);
	
	
	glEnd();




	glColor3f(r,g,b);

	// front
	glBegin(GL_TRIANGLE_FAN);
	
	glTexCoord2f(frame->u[3],frame->v[3]);
	glVertex3f(-frame->ox+frame->width,-frame->oy+frame->height,frontZ);

	glTexCoord2f(frame->u[2],frame->v[2]);
	glVertex3f(-frame->ox,-frame->oy+frame->height,frontZ);

	glTexCoord2f(frame->u[1],frame->v[1]);
	glVertex3f(-frame->ox,-frame->oy,frontZ);

	glTexCoord2f(frame->u[0],frame->v[0]);
	glVertex3f(-frame->ox+frame->width,-frame->oy,frontZ);	
	
	glEnd();



	glPopMatrix();
}