
// ------------------------------------------------------------------------------
//
// LudumDare.com 48hr compo, July 2002
//
// Dave Reed (Bluescreen)'s entry
//
// OpenGL init code based on the NeHe turorials (nehe.gamedev.net)
//
// DevIL init code added
//
// ------------------------------------------------------------------------------

#include	<windows.h>
#include	<gl\gl.h>			// Include OpenGL

#include	"il\il.h"			// Include DevIL
#include	"il\ilu.h"
#include	"il\ilut.h"
#include	"fmod.h"


#include	"winmain.h"
#include	"game.h"

#define		APP_NAME "The flame game!"

#ifdef NDEBUG
bool        g_bFullScreen=TRUE;
#else
bool        g_bFullScreen=FALSE;
#endif

bool		g_bActive=FALSE;
bool		g_keyState[256];
int         g_xRes=640,g_yRes=480,g_BPP=16;

int         texNames[256];

int			g_listBase;

HDC			hDC=NULL;		
HGLRC		hRC=NULL;		
HWND		hWnd=NULL;		
HINSTANCE	hInstance;

LRESULT	CALLBACK WndProc(HWND,UINT,WPARAM,LPARAM);

// ------------------------------------------------------------------------------
// GL_deinit - Attempt to shut down OpenGL
// ------------------------------------------------------------------------------
void GL_deinit()
{
	if (g_bFullScreen)							
	{
		ChangeDisplaySettings(NULL,0);			
		ShowCursor(TRUE);						
	}

	if (hRC)									
	{
		wglMakeCurrent(NULL,NULL);
		wglDeleteContext(hRC);		
		hRC=NULL;			
	}

	if (hDC && !ReleaseDC(hWnd,hDC))		
	{		
		hDC=NULL;
	}

	if (hWnd && !DestroyWindow(hWnd))
	{		
		hWnd=NULL;								
	}

	if (!UnregisterClass("OpenGL",hInstance))	
	{		
		hInstance=NULL;
	}
}

// ------------------------------------------------------------------------------
// GL_init - Create the OpenGL window
// ------------------------------------------------------------------------------
BOOL GL_init(char* title, int width, int height, int bits, bool fullscreenflag)
{
	GLuint		PixelFormat;	
	WNDCLASS	wc;				
	DWORD		dwExStyle;		
	DWORD		dwStyle;		
	RECT		WindowRect;		

	WindowRect.left=(long)0;		
	WindowRect.right=(long)width;	
	WindowRect.top=(long)0;			
	WindowRect.bottom=(long)height;	

	g_bFullScreen=fullscreenflag;		

	hInstance			= GetModuleHandle(NULL);				
	wc.style			= CS_HREDRAW | CS_VREDRAW | CS_OWNDC;	
	wc.lpfnWndProc		= (WNDPROC) WndProc;					
	wc.cbClsExtra		= 0;									
	wc.cbWndExtra		= 0;									
	wc.hInstance		= hInstance;							
	wc.hIcon			= LoadIcon(NULL, IDI_WINLOGO);			
	wc.hCursor			= LoadCursor(NULL, IDC_ARROW);			
	wc.hbrBackground	= NULL;									
	wc.lpszMenuName		= NULL;									
	wc.lpszClassName	= "OpenGL";								

	// Never seen this fail before...
	if (!RegisterClass(&wc)) exit(0);
	
	// Attempt to go fullscreen
	if (g_bFullScreen)			
	{
		DEVMODE dmScreenSettings;							
		memset(&dmScreenSettings,0,sizeof(dmScreenSettings));
		dmScreenSettings.dmSize=sizeof(dmScreenSettings);	
		dmScreenSettings.dmPelsWidth	= width;			
		dmScreenSettings.dmPelsHeight	= height;			
		dmScreenSettings.dmBitsPerPel	= bits;				
		dmScreenSettings.dmFields=DM_BITSPERPEL|DM_PELSWIDTH|DM_PELSHEIGHT;

		// Fall back to windowed if mode cahnge fails
		if (ChangeDisplaySettings(&dmScreenSettings,CDS_FULLSCREEN)!=DISP_CHANGE_SUCCESSFUL) g_bFullScreen=false;		
	}

	// Set window style for FS/windowed
	if (g_bFullScreen)										
	{
		dwExStyle=WS_EX_APPWINDOW;							
		dwStyle=WS_POPUP;									
		ShowCursor(FALSE);									
	}
	else
	{
		dwExStyle=WS_EX_APPWINDOW | WS_EX_WINDOWEDGE;		
		dwStyle=WS_MINIMIZEBOX|WS_OVERLAPPED|WS_SYSMENU; //WS_OVERLAPPEDWINDOW;						
	}

	AdjustWindowRectEx(&WindowRect, dwStyle, FALSE, dwExStyle);		// Adjust Window To True Requested Size

	// Create The Window, quit if it fails
	if (!(hWnd=CreateWindowEx(	dwExStyle,"OpenGL",title, dwStyle | WS_CLIPSIBLINGS |	WS_CLIPCHILDREN,		
								0, 0, (WindowRect.right-WindowRect.left),(WindowRect.bottom-WindowRect.top),	
								NULL,NULL,hInstance,NULL)))	exit(0);
		

	// Finally, we get to do some GL stuff...
	static	PIXELFORMATDESCRIPTOR pfd=				
	{
		sizeof(PIXELFORMATDESCRIPTOR),				
		1, PFD_DRAW_TO_WINDOW | PFD_SUPPORT_OPENGL | PFD_DOUBLEBUFFER, 
		PFD_TYPE_RGBA, bits, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,								
		16,	0, 0, PFD_MAIN_PLANE, 0, 0, 0, 0				
	};
	
	if (!(hDC=GetDC(hWnd)))							// Did We Get A Device Context?
	{	
		MessageBox(NULL,"Can't Create A GL Device Context.","ERROR",MB_OK|MB_ICONEXCLAMATION);
		return FALSE;					
	}

	if (!(PixelFormat=ChoosePixelFormat(hDC,&pfd)))	// Did Windows Find A Matching Pixel Format?
	{		
		MessageBox(NULL,"Can't Find A Suitable PixelFormat.","ERROR",MB_OK|MB_ICONEXCLAMATION);
		return FALSE;								// Return FALSE
	}

	if(!SetPixelFormat(hDC,PixelFormat,&pfd))		// Are We Able To Set The Pixel Format?
	{		
		MessageBox(NULL,"Can't Set The PixelFormat.","ERROR",MB_OK|MB_ICONEXCLAMATION);
		return FALSE;								// Return FALSE
	}

	if (!(hRC=wglCreateContext(hDC)))				// Are We Able To Get A Rendering Context?
	{		
		MessageBox(NULL,"Can't Create A GL Rendering Context.","ERROR",MB_OK|MB_ICONEXCLAMATION);
		return FALSE;								// Return FALSE
	}

	if(!wglMakeCurrent(hDC,hRC))					// Try To Activate The Rendering Context
	{	
		MessageBox(NULL,"Can't Activate The GL Rendering Context.","ERROR",MB_OK|MB_ICONEXCLAMATION);
		return FALSE;								// Return FALSE
	}

	ShowWindow(hWnd,SW_SHOW);						// Show The Window
	SetForegroundWindow(hWnd);						// Slightly Higher Priority
	SetFocus(hWnd);									// Sets Keyboard Focus To The Window
	

	// Init render state	
	glClearColor(0.0f, 0.0f, 0.2f, 0.5f);			
	glClearDepth(1.0f);								
	glEnable(GL_DEPTH_TEST);						
	glDepthFunc(GL_LEQUAL);				
	
	// Init viewport
	glViewport(0,0,width,height);					

	glMatrixMode(GL_PROJECTION);				
	glLoadIdentity();									
	gluPerspective(45.0f,(GLfloat)width/(GLfloat)height,0.1f,100.0f);

	glMatrixMode(GL_MODELVIEW);							
	glLoadIdentity();

	glGenTextures(256,(unsigned int *)(&texNames[0]));


	// Create a font (from another NeHe tutorial)
	{
		HFONT	font;					
		HFONT	oldfont;				

		font = CreateFont(	-18,0,0,0,	FW_BOLD,FALSE,	FALSE,	FALSE, ANSI_CHARSET,			
							OUT_TT_PRECIS, CLIP_DEFAULT_PRECIS,	ANTIALIASED_QUALITY, FF_DONTCARE|DEFAULT_PITCH,		
							"Comic Sans MS");					

		oldfont = (HFONT)SelectObject(hDC, font);    

		g_listBase=glGenLists(96);			
		wglUseFontBitmaps (hDC, 32,96,g_listBase); 

		SelectObject(hDC, oldfont);					
		DeleteObject(font);							
	}
		

	
	return TRUE;						
}



// ------------------------------------------------------------------------------
// WndProc() - Window callback
// ------------------------------------------------------------------------------

LRESULT CALLBACK WndProc(HWND hWnd,UINT	uMsg,WPARAM	wParam,LPARAM	lParam)		
{
	switch (uMsg)																// Check For Windows Messages
	{
		case WM_ACTIVATE:								
			if (!HIWORD(wParam))												// Check Max-Minimization State			
				g_bActive=true;		
			else			
				g_bActive=false;			
			return 0;							

		case WM_SYSCOMMAND:														// Intercept System Commands		
			switch (wParam)
			{
				case SC_SCREENSAVE:				
				case SC_MONITORPOWER:			
				return 0;						
			}
			break;										

		case WM_CLOSE:															// Did We Receive A Close Message?		
			PostQuitMessage(0);													// Send A Quit Message
			return 0;						

		case WM_KEYDOWN:														// Is A Key Being Held Down?		
			g_keyState[wParam] = true;				
			return 0;								

		case WM_KEYUP:															// Has A Key Been Released?
			g_keyState[wParam] = false;				
			return 0;								
		
	}
	
	return DefWindowProc(hWnd,uMsg,wParam,lParam);
}


// ------------------------------------------------------------------------------
// WinMain() - message loop, basic init code
// ------------------------------------------------------------------------------

int WINAPI WinMain(HINSTANCE hInstance,HINSTANCE hPrevInstance,LPSTR lpCmd,int nCmdShow)		
{
	MSG		msg;									
	bool	bQuit=FALSE;								


	if (!GL_init(APP_NAME,g_xRes,g_yRes,g_BPP,g_bFullScreen)) return 1;			// Init OpenGL

	
	ilInit();
	iluInit();
	ilutRenderer(ILUT_OPENGL);

	FSOUND_Init(44100, 32, FSOUND_INIT_USEDEFAULTMIDISYNTH);

	GAME_init();
	GAME_initTextures();

	

	while(!bQuit)									
	{
		if (PeekMessage(&msg,NULL,0,0,PM_REMOVE))	
		{
			if (msg.message==WM_QUIT)			
			{
				bQuit=TRUE;							
			}
			else									
			{
				TranslateMessage(&msg);				
				DispatchMessage(&msg);				
			}
		}
		else										
		{			
			if (g_bActive)														// If app ain't minimized, run a frame
			{
				GAME_update();
				GAME_render();
				
 				SwapBuffers(hDC);										
			}

			if (g_keyState[VK_ESCAPE]) bQuit=TRUE;												
			
			if (g_keyState[VK_F1])												// Handle fullscreen-windowed switching
			{
				g_keyState[VK_F1]=false;			
				
				GL_deinit();
				g_bFullScreen=!g_bFullScreen;		
								
				if (!GL_init(APP_NAME,g_xRes,g_yRes,g_BPP,g_bFullScreen)) return 1;								
				
				ilShutDown();
				ilInit();
				iluInit();
				ilutRenderer(ILUT_OPENGL);
				GAME_initTextures();
			}
		}
	}

	GAME_deinit();
	FSOUND_Close();

	GL_deinit();																// Shutdown

	return 0;
}




// ------------------------------------------------------------------------------
// I'm starting to like DevIL :)
// ------------------------------------------------------------------------------

void GL_bindTexture(int tex)
{
	glBindTexture(GL_TEXTURE_2D,texNames[tex]);
}


char tex[64][64][3];

int GL_loadTexture(int tex,char *filename)
{	
	int r;

	
	texNames[tex]=ilutGLLoadImage(filename);
	GL_bindTexture(tex);	

	return r;
}


void GL_print(char *string, float x,float y)
{
	glDisable(GL_TEXTURE_2D);

	glLoadIdentity();		
	glColor3f(0.0f,0.0f,0.0f);
	glTranslatef(0.0045f,-0.0045f,-1.0f);	
	glRasterPos2f(-0.54f+x,0.35-y);	
	glListBase(g_listBase - 32);	
	// now draw the characters in a string 
	glCallLists (strlen(string), GL_UNSIGNED_BYTE, string); 
		
	glLoadIdentity();
	glColor3f(2.0f,1.0f,0.0f);
	glTranslatef(0.0f,0.0f,-1.0f);	
	glRasterPos2f(-0.54f+x,0.35-y);	
	glPushAttrib(GL_LIST_BIT);
	glListBase(g_listBase - 32);	
	// now draw the characters in a string 
	glCallLists (strlen(string), GL_UNSIGNED_BYTE, string); 

	
	glPopAttrib();
}
			


FSOUND_SAMPLE *sfx[32];

void SFX_load(int id,char *filename)
{
	sfx[id] = FSOUND_Sample_Load(FSOUND_UNMANAGED, filename, FSOUND_NORMAL | FSOUND_HW2D, 0);
	FSOUND_Sample_SetLoopMode(sfx[id], FSOUND_LOOP_OFF);	
}


void SFX_play(int id)
{
	FSOUND_PlaySound(FSOUND_FREE, sfx[id]);
}


void SFX_free(int id)
{
	FSOUND_Sample_Free(sfx[id]);
}